﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Documents;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Shapes;


namespace CustomControlSample
{
    [TemplatePart(Name = ThumbPartName, Type = typeof (Thumb))]
    [TemplatePart(Name = RectanglePartName, Type = typeof (Rectangle))]
    public sealed class SimpleSlider : Control
    {
        private const string ThumbPartName = "PART_Thumb";
        private const string RectanglePartName = "PART_Rectangle";

        private Thumb _thumb;
        private Rectangle _rectangle;

        /// <summary>
        /// 기본생성자
        /// </summary>
        public SimpleSlider()
        {
            this.DefaultStyleKey = typeof (SimpleSlider);
        }

        /// <summary>
        /// Value
        /// </summary>
        public double Value
        {
            get { return (double) GetValue(ValueProperty); }
            set { SetValue(ValueProperty, value); }
        }

        /// <summary>
        /// Value DP
        /// </summary>
        public static readonly DependencyProperty ValueProperty =
            DependencyProperty.Register("Value", typeof (double), typeof (SimpleSlider),
                new PropertyMetadata(0.0, OnValueChanged));

        /// <summary>
        /// Value DP Changed
        /// </summary>
        /// <param name="d"></param>
        /// <param name="e"></param>
        private static void OnValueChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            var customSlider = (SimpleSlider) d;
            customSlider.UpdateControls();
        }

        /// <summary>
        /// Maximum
        /// </summary>
        public double Maximum
        {
            get { return (double) GetValue(MaximumProperty); }
            set { SetValue(MaximumProperty, value); }
        }

        /// <summary>
        /// Maximum DP
        /// </summary>
        public static readonly DependencyProperty MaximumProperty =
            DependencyProperty.Register("Maximum", typeof (double), typeof (SimpleSlider), new PropertyMetadata(100.0));

        /// <summary>
        /// Minimum
        /// </summary>
        public double Minimum
        {
            get { return (double) GetValue(MinimumProperty); }
            set { SetValue(MinimumProperty, value); }
        }

        /// <summary>
        /// Minimum DP
        /// </summary>
        public static readonly DependencyProperty MinimumProperty =
            DependencyProperty.Register("Minimum", typeof (double), typeof (SimpleSlider), new PropertyMetadata(0.0));

        /// <summary>
        /// 템플릿이 적용될 때 실행
        /// </summary>
        protected override void OnApplyTemplate()
        {
            base.OnApplyTemplate();
            //이름으로 각 파트 컨트롤 땡김
            _thumb = GetTemplateChild(ThumbPartName) as Thumb;
            if (_thumb == null) return;
            //이벤트 연결
            _thumb.DragDelta += _thumb_DragDelta;
            _rectangle = GetTemplateChild(RectanglePartName) as Rectangle;
            //사이즈 체인지 이벤트 연결
            SizeChanged += SimpleSlider_SizeChanged;
            //IsEnabled 체인지 이벤트 연결
            IsEnabledChanged += SimpleSlider_IsEnabledChanged;
        }
        /// <summary>
        /// IsEnabed 변경 이벤트 처리
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void SimpleSlider_IsEnabledChanged(object sender, DependencyPropertyChangedEventArgs e)
        {
            VisualStateManager.GoToState(this, IsEnabled ? "NormalState" : "DisabledState", true);
        }

        /// <summary>
        /// 사이즈 체인지 이벤트 처리
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void SimpleSlider_SizeChanged(object sender, SizeChangedEventArgs e)
        {
            if (Math.Abs(e.NewSize.Width - e.PreviousSize.Width) > 0)
            {
                UpdateControls();
            }
        }

        /// <summary>
        /// 썸 드레그 처리
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void _thumb_DragDelta(object sender, DragDeltaEventArgs e)
        {
            var pixelDiff = e.HorizontalChange;
            var currentLeft = Canvas.GetLeft(_thumb);

            // trying to drag too far left
            if ((currentLeft + pixelDiff) < 0)
            {
                Value = 0;
            }
            // trying to drag too far right
            else if ((currentLeft + pixelDiff + _thumb.ActualWidth) > ActualWidth)
            {
                Value = Maximum;
            }
            else
            {
                //슬라이더 실제 사이즈
                var totalSize = ActualWidth;
                //현재 위치/전체 사이즈
                var ratioDiff = pixelDiff/totalSize;
                //최대값 - 최소값
                var rangeSize = Maximum - Minimum;
                //최종 증감 값 계산
                var rangeDiff = rangeSize*ratioDiff;
                Value += rangeDiff;
            }
        }

        /// <summary>
        /// Updates the controls.
        /// </summary>
        private void UpdateControls()
        {
            double halfTheThumbWith;

            if (_thumb == null) return;
            //썸 실제크기 절반
            halfTheThumbWith = _thumb.ActualWidth/2;
            //컨트롤 실제 너비에서 썸을 뺀걸 전체 사이즈로
            double totalSize = ActualWidth - halfTheThumbWith*2;
            //%구함
            double ratio = totalSize/(Maximum - Minimum);

            if (_thumb == null) return;
            //썸 위치 이동
            Canvas.SetLeft(_thumb, ratio*Value);

            if (_rectangle == null) return;
            //이동 후 남은 곳에 노란색으로...
            _rectangle.Width = ratio*Value + halfTheThumbWith;
        }
    }
}